#!/usr/bin/perl

# suprnova light 2 - create your own suprnova mirror
# Copyright (C) 2004 ntworks.net
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details:
# http://www.gnu.org/copyleft/gpl.html
#
# snmirror.pl is part of the suprnova light 2 scripts.
# Get the latest version at http://suprnova.ntworks.net/
# Send patches and comments to snlight2@ntworks.net

use strict;
use Getopt::Std;
use Time::HiRes qw (gettimeofday tv_interval);
use POSIX qw (strftime);
use CGI;
use LWP::UserAgent;

$| = 1;

my ($currentVersion, $latestVersion) = (20041220, 0);

my %cmdln = getCommandline ();
my %conf = getConfiguration ($cmdln {'c'});
my %catConf = getCategoryConfiguration ();
my %catNames;

if ($conf {'cgimode'} eq 'yes') {
    print "Content-type: text/plain\n\n";
    %cmdln = getCGIParams ();
}

if ($conf {'updatecheck'} eq 'yes') {
    $latestVersion = checkSNLLatestVersion ();
    if (($latestVersion > $currentVersion) and ($conf {'output'} eq 'yes')) {
        print "there is a new version available at http://suprnova.ntworks.net/\n";
    }
}

my $mirror = getMirror ();
my $template = getTemplate ();

createNews ();

sub dieNice {
    my $text = shift;
    if ($conf {'cgimode'} eq 'yes') {
        print $text;
        exit;
    }
    else { die $text }
}

sub getCGIParams {
    my $query = new CGI;
    my (%params, %tmp);
    $tmp {'n'} = $query -> param ('n');
    $tmp {'m'} = $query -> param ('m');
    $tmp {'b'} = $query -> param ('b');
    $params {'n'} = 1 if (defined $tmp {'n'} || exists $cmdln {'n'});
    $params {'b'} = 1 if (defined $tmp {'b'} || exists $cmdln {'b'});
    $params {'m'} = $tmp {'m'} if defined $tmp {'m'};
    $params {'m'} = $cmdln {'m'} if exists $cmdln {'m'};
    %params;
}

sub savePage {
    my ($path, $content, $type) = @_;
    my $page = $template;
    my $updString = ($type == 1) ? $conf {'newtorrentsupdates'} : $conf {'categoriesupdates'};
    my $generated = strftime ('%d-%m-%Y, <b>%H:%M:%S</b> ', localtime (time ())) . $conf {'timezone'};
    $page =~ s|<!--CONTENT-->|$content|sg;
    $page =~ s|<!--GENERATED-->|updated $updString<br />generated at $generated|sg;
    if ($conf {'jssort'} eq 'yes') {
        $page =~ s|<!--JSCRIPT-->|<script src="tablesort.js" type="text/javascript"></script>|sg
    }
    else { $page =~ s|<!--JSCRIPT-->||sg }
    open (OUT, "> $path") || dieNice "could not open '$path' for output: $!\n";
    print OUT $page;
    close OUT;
}

sub getTorrentServers {
    my $page = shift || return undef;
    my %servers;
    while ($page =~ /torrent_servers\[([-0-9]+?)\] = '(.+?)'/isg) { $servers {$1} = $2 }
    %servers;
}

sub parseNews {
    my $newsPage = shift || return undef;
    my ($oldCategory, $catNumber, $catLinks, $table, $cn) = ('', 0, '', '', 0);
    my %torrentServers = getTorrentServers ($newsPage);
    my ($d, $s) = ('\d+', "[^']*");
    while ($newsPage =~ /(at\d?\(($d),'($s)',($d),($d),'($s)','($s)','($s)','($s)','($s)',($d),($d),'($s)','($s)','($s)','($s)','($s)',($d,){8}($d)\))/oisg) {
        my ($mainCategory, $subCategory) = split (/ - /, $7, 2);
        if ($mainCategory ne $oldCategory) {
            $table .= "</tbody></table>" if ($cn != 0);
            if ($conf {'jssort'} eq 'yes') {
                ($table .= <<"                    _EOT_") =~ s/^\s+//gm;
                    <table class="torrents"><thead>
                    <tr class="header"><th colspan="6" class="category"><a name="l$catNumber" id="l$catNumber"></a>
                    $mainCategory</th><th class="toplink"><a href="#top">top</a></th></tr><tr class="header">
                    <td class="hntime" onclick="sortTable('torrents$cn',0)" title="click to order by time">Since
                    <img alt="ns" id="img_torrents$cn\_0" src="nosort.gif" /></td>
                    <td class="hntorrent" onclick="sortTable('torrents$cn',1)" title="click to order by name">Torrents
                    <img alt="ns" id="img_torrents$cn\_1" src="nosort.gif" /></td>
                    <td class="hnsize" onclick="sortTable('torrents$cn',2)" title="click to order by size">Filesize
                    <img alt="ns" id="img_torrents$cn\_2" src="nosort.gif" /></td>
                    <td class="hnseeds" onclick="sortTable('torrents$cn',3)" title="click to order by seeds">Seeds
                    <img alt="ns" id="img_torrents$cn\_3" src="nosort.gif" /></td>
                    <td class="hndls" onclick="sortTable('torrents$cn',4)" title="click to order by downloads">DLs
                    <img alt="ns" id="img_torrents$cn\_4" src="nosort.gif" /></td>
                    <td class="hnsubcat" onclick="sortTable('torrents$cn',5)" title="click to order by category">Category
                    <img alt="ns" id="img_torrents$cn\_5" src="nosort.gif" /></td>
                    <td class="hnkind" onclick="sortTable('torrents$cn',6)" title="click to order by type">Type
                    <img alt="ns" id="img_torrents$cn\_6" src="nosort.gif" /></td></tr></thead>
                    <tbody id="torrents$cn">
                    _EOT_
            }
            else {
                ($table .= <<"                    _EOT_") =~ s/^\s+//gm;
                    <table class="torrents"><thead>
                    <tr class="header"><th colspan="6" class="category"><a name="l$catNumber" id="l$catNumber"></a>
                    $mainCategory</th><th class="toplink"><a href="#top">top</a></th></tr><tr class="header">
                    <td class="hntime">Since</td><td class="hntorrent">Torrents</td><td class="hnsize">Filesize</td>
                    <td class="hnseeds">Seeds</td><td class="hndls">DLs</td><td class="hnsubcat">Category</td>
                    <td class="hnkind">Type</td></tr></thead><tbody id="torrents$cn">
                    _EOT_
            }
            $catLinks .= '<a href="#l' . $catNumber . '" class="catLink">' . $mainCategory . '</a> ';
            $oldCategory = $mainCategory;
            $catNumber++;
            $cn++;
        }
        my ($currentURL, $aStyle, $ahref, $shortName);
        if (length ($8) > 45) { $shortName = substr ($8, 0, 45) . '...' }
        else { $shortName = $8 }
        if ($torrentServers {$4} ne '') {
            $currentURL = $torrentServers {$4} . '/torrents/' . $9;
            $ahref = "<a href=\"dl.cgi?$currentURL\" title=\"$8\">$shortName</a>";
        }
        else {
            $currentURL = "javascript:alert('no known mirror')";
            $aStyle = 'style="text-decoration:line-through;"';
            $ahref = "<span title=\"no known mirror: $8\" $aStyle>$shortName</span>";
        }
        my ($seeds, $shortSubCat, $regImg, $infoLnk);
        if ($19 eq '1') { $regImg = "<img src=\"reg.gif\" title=\"tracker requires registration\" alt=\"reg\" /> " }
        else { $regImg = "<img src=\"space.gif\" title=\"\" alt=\"\" /> " }
        if ($17 ne '') { $infoLnk = "<a href=\"$17\"><img src=\"info.gif\" title=\"info\" alt=\"info\" /></a> "; }
        else { $infoLnk = "<img src=\"space.gif\" title=\"\" alt=\"\" /> " }
        if (length ($subCategory) > 12) { $shortSubCat = substr ($subCategory, 0, 12) . '...' }
        else { $shortSubCat = $subCategory }
        if ($11 == 0) { $seeds = '<span class="red">0</span>' }
        else { $seeds = '<span class="green">' . $11 . '</span>' }
        ($table .= <<"            _EOT_") =~ s/^\s+//gm;
            <tr><td class="ntime">$3</td><td class="ntorrent">$regImg$infoLnk&nbsp;&nbsp;$ahref</td>
            <td class="nsize">$10</td><td class="nseeds">$seeds</td><td class="ndls">$12</td>
            <td class="nsubcat">$shortSubCat</td><td class="nkind">$13</td></tr>
            _EOT_
    }
    $table .= '</tbody></table>';
    ($table, $catLinks);
}

sub getCategories {
    my $newsPage = shift || return undef;
    my ($numberCategories, $numberTorrents) = (0, 0);
    my (%allSubCategories, %allCategories);
    while ($newsPage =~ /<a href="([^"]*\/list_torrents\/([^"]*)\.html)">/sgi) {
        my $currentTopCatPage = getPage ($1);
        my $currentTopCatName = ucfirst ($2);
        while ($currentTopCatPage =~ /JavaScript:showTbl\('([^']*)','([^']*)'\)/sgi)  {
            my ($urlPart, $currentSubCatName) = ($1, $2);
            $urlPart =~ /[^\d]*(\d*)$/sgi;
            my $currentID = sprintf '%03s', $1;
            $catNames {$currentID} = "$currentTopCatName - $currentSubCatName";
            if ($catConf {$currentID} == 1) {
                $allSubCategories {"$urlPart.html"} = $currentSubCatName
            }
        }
    }
    foreach my $url (keys %allSubCategories) {
        my $currentSubCatPage = getPage ($url);
        $url =~ /\/(\d+)\.html$/i;
        my $currentFileName = "$1.htm";
        my %currentServers = getTorrentServers ($currentSubCatPage);
        my $currentContent;
        if ($conf {'jssort'} eq 'yes') {
            ($currentContent = <<"                _EOT_") =~ s/^\s+//gm;
                <table class="torrents"><thead><tr class="header"><th colspan="10" class="category"><!--CATEGORY--></th>
                </tr><tr class="header">
                <td class="htime" onclick="sortTable('torrents',0)" title="click to order by date">Since
                <img alt="ns" id="img_torrents_0" src="nosort.gif" /></td>
                <td class="htorrent" onclick="sortTable('torrents',1)" title="click to order by name">Torrents
                <img alt="ns" id="img_torrents_1" src="nosort.gif" /></td>
                <td class="hsize" onclick="sortTable('torrents',2)" title="click to order by size">Filesize
                <img alt="ns" id="img_torrents_2" src="nosort.gif" /></td>
                <td class="hseeds" onclick="sortTable('torrents',3)" title="click to order by seeds">Seeds
                <img alt="ns" id="img_torrents_3" src="nosort.gif" /></td>
                <td class="hdls" onclick="sortTable('torrents',4)" title="click to order by downloads">DLs
                <img alt="ns" id="img_torrents_4" src="nosort.gif" /></td>
                <td class="hkind" onclick="sortTable('torrents',5)" title="click to order by type">Type
                <img alt="ns" id="img_torrents_5" src="nosort.gif" /></td>
                </tr></thead><tbody id="torrents">
                _EOT_
        }
        else {
            ($currentContent = <<"                _EOT_") =~ s/^\s+//gm;
                <table class="torrents"><thead><tr class="header"><th colspan="10" class="category"><!--CATEGORY--></th>
                </tr><tr class="header"><td class="htime">Since</td><td class="htorrent">Torrents</td>
                <td class="hsize">Filesize</td><td class="hseeds">Seeds</td><td class="hdls">DLs</td>
                <td class="hkind">Type</td></tr></thead><tbody id="torrents">
                _EOT_
        }
        my ($currentCategory, $currentNumber, $d, $s) = ('', 0, '\d+', "[^']*");
        while ($currentSubCatPage =~ /(at\d?\(($d),'($s)',($d),($d),'($s)','($s)','($s)','($s)','($s)',($d),($d),'($s)','($s)','($s)','($s)','($s)',($d,){8}$d\))/oisg) {
            my ($currentURL, $aStyle, $ahref, $shortName);
            if (length ($8) > 70) { $shortName = substr ($8, 0, 70) . '...' }
            else { $shortName = $8 }
            if ($currentServers {$4} ne '') {
                $currentURL = $currentServers {$4} . '/torrents/' . $9;
                $ahref = "<a href=\"dl.cgi?$currentURL\" title=\"$8\">$shortName</a>";
            }
            else {
                $currentURL = "javascript:alert('no known mirror')";
                $aStyle = ' style="text-decoration:line-through;"';
                $ahref = "<span title=\"no known mirror: $8\" $aStyle>$shortName</span>";
            }
            my ($seeds, $regImg, $infoLnk);
            if ($11 == 0) { $seeds = '<span class="red">0</span>' }
            else { $seeds = '<span class="green">' . $11 . '</span>' }
            if ($19 eq '1') { $regImg = "<img src=\"reg.gif\" title=\"tracker requires registration\" alt=\"reg\" /> " }
            else { $regImg = "<img src=\"space.gif\" title=\"\" alt=\"\" /> " }
            if ($17 ne '') { $infoLnk = "<a href=\"$17\"><img src=\"info.gif\" title=\"info\" alt=\"info\" /></a> "; }
            else { $infoLnk = "<img src=\"space.gif\" title=\"\" alt=\"\" /> " }
            $currentContent .=
                "<tr><td class=\"time\">$3</td>" .
                "<td class=\"torrent\">$regImg$infoLnk&nbsp;&nbsp;$ahref</td><td class=\"size\">$10</td><td class=\"seeds\">$seeds</td>" .
                "<td class=\"dls\">$12</td><td class=\"kind\">$13</td></tr>\n";
            $currentCategory = $7;
            $currentNumber++;
            $numberTorrents++;
        }
        $currentContent .= '<tr><td class="notorrents" colspan="10">no torrents found</td></tr>'
            if $currentNumber == 0;
        $currentContent .= '</tbody></table>';
        $currentContent =~ s/<!--CATEGORY-->/$currentCategory/g;
        savePage ("$conf{htmldir}/$currentFileName", $currentContent);
        my ($currentMainCat, $currentSubCat) = split (/ - /, $currentCategory, 2);
        ${$allCategories{$currentMainCat}} {$currentSubCat} = $currentFileName;
    }
    my $rowNumber = 0;
    my (@name, @isHeader, @url, $mainCategory, $subCategory);
    open (CAT, "> $conf{basedir}/$conf{categoryfile}") || dieNice "could not open $conf{categoryfile}: $!\n";
    foreach $mainCategory (sort (keys %allCategories)) {
        unless ($mainCategory eq '') {
            print CAT "$mainCategory\t";
            $name [$rowNumber] = $mainCategory;
            $isHeader [$rowNumber] = 1;
            $url [$rowNumber] = '';
            $rowNumber++;
            foreach $subCategory (sort (keys %{$allCategories{$mainCategory}})) {
                unless ($subCategory eq '') {
                    $numberCategories++;
                    my $currentURL = ${$allCategories{$mainCategory}} {$subCategory};
                    print CAT substr ($currentURL, 0, -4) . ' ';
                    $name [$rowNumber] = $subCategory;
                    $url [$rowNumber] = $currentURL;
                    $isHeader [$rowNumber] = 0;
                    $rowNumber++;
                }
            }
            print CAT "\n";
            $name [$rowNumber] = '';
            $isHeader [$rowNumber] = 1;
            $url [$rowNumber] = '';
            $rowNumber++;
        }
    }
    close CAT;
    $rowNumber = $rowNumber - 2;
    my $tblRows = div ($rowNumber, 4) + 1;
    my $content = '<table class="categories">';
    my ($currentRow, $currentColumn);
    for ($currentRow = 1; $currentRow <= $tblRows; $currentRow++) {
        $content .= '<tr>';
        for ($currentColumn = 1; $currentColumn <= 4; $currentColumn++) {
            my $currentEntry = $currentRow + (($currentColumn-1) * $tblRows) - 1;
            my ($shortName, $class);
            $shortName = (length ($name [$currentEntry]) > 25) ?
                substr ($name [$currentEntry], 0, 25) . '...' : $name [$currentEntry];
            if ($currentEntry <= $rowNumber) {
                if ($isHeader [$currentEntry] == 1) {
                    if ($shortName ne '') {
                        $shortName .= ':';
                        $class = 'header';
                    }
                    else { $class = 'space' }
                    $content .= "<td class=\"$class\">$shortName</td>";
                }
                else {
                    $content .= '<td class="category">' .
                        '<a href="' . $url [$currentEntry] . '">' . $shortName . '</a></td>';
                }
            }
            else { $content .= '<td>&nbsp;</td>' }
        }
        $content .= '</tr>';
    }
    $content .= '</table>';
    savePage ("$conf{htmldir}/categories.htm", $content);
    updateCategoryConfiguration ();
    ($numberCategories, $numberTorrents);
}

sub updateCategoryConfiguration {
    open (CATCONF, "> $conf{categoryconf}") || dieNice ("could not open configuration file: $!\n");
    my @keys = sort { $catNames{$a} cmp $catNames{$b} } keys %catNames;
    print CATCONF "# Use this file to specify which category should be mirrored (=1) or\n";
    print CATCONF "# not. This file will be updated on each execution of the script. If\n";
    print CATCONF "# a new category is found, the default is to not mirror this one, you\n";
    print CATCONF "# have to edit this file and set the appropriate entry to 1 if you\n";
    print CATCONF "# want to include it.\n#\n";
    print CATCONF "# Please note that this affects only the category pages, on the new\n";
    print CATCONF "# torrent pages all categories will be listed.\n\n";
    foreach (@keys) {
        if (($catConf {$_} eq '') || (! defined $catConf {$_}) || (! exists $catConf {$_})) {
            $catConf {$_} = 0;
        }
        print CATCONF "$_=$catConf{$_}\t# $catNames{$_}\n";
    }
    close CATCONF;
}

sub div {
    my $a = shift || return undef;
    my $b = shift || return undef;
    return ($a - ($a % $b)) / $b;
}

sub createNews {
    my $newsPage = getPage ($mirror . 'list_news.html');
    my ($numberCategories, $numberTorrents, $info, %re);
    ($numberCategories, $numberTorrents) = getCategories ($newsPage) unless exists $cmdln {'n'};
    my $tries = 1;
    my ($torrentsToday, $torrentsYesterday) = (0, 0);
    while (length ($newsPage) < 4000 && $tries <= 10) {
        $mirror = backupMirror ("$conf{basedir}/$conf{mirrorfile}");
        $newsPage = getPage ($mirror . 'list_news.html');
        $tries++;
    }
    dieNice ("unable to retrieve the news page\n") if length ($newsPage) < 4000;
    my $reSeperator = "<div class='seperator'>";
    $newsPage =~ m|<br>Generated @ ([^<]*)<br>.*?$reSeperator([^<]*)</div>.*?$reSeperator([^<]*)</div>|sgi;
    my ($newsCreated, $newsDateToday, $newsDateYesterday) = ($1, $2, $3);
    (my $output = <<"        _EOT_") =~ s/^\s+//gm;
        <table class="date"><tr><td><table><tr><td class="date">$newsDateToday</td>
        <td class="created">original page created at $newsCreated</td></tr></table></td></tr><tr><td>
        <table><tr><td class="categories"><!--CATLINKS--></td><td class="yesterday">
        <a href="yesterday.htm" class="catLink">&gt;&gt;Yesterday&lt;&lt;</a></td></tr></table></td></tr>
        </table>
        _EOT_
    my ($newsTable, $categoryLinks) = parseNews ($newsPage);
    $output .= $newsTable;
    ($output .= <<"        _EOT_") =~ s/^\s+//gm;
        <table class="date"><tr><td class="date">
            $newsDateYesterday <a href="yesterday.htm" class="date">show torrents</a>
        </td></tr></table>
        _EOT_
    $output =~ s/<!--CATLINKS-->/$categoryLinks/sg;
    savePage ("$conf{htmldir}/index.htm", $output, 1);
    $torrentsToday = () = $output =~ /<a href="[^"]+\.torrent"/g;
    $newsPage = getPage ($mirror . 'list_news_1.html');
    ($output = <<"        _EOT_") =~ s/^\s+//gm;
        <table class="date"><tr><td><table><tr><td class="date">$newsDateYesterday</td></tr></table></td>
        </tr><tr><td><table><tr><td class="categories"><!--CATLINKS--></td><td class="yesterday">
        <a href="index.htm" class="catLink">&gt;&gt;Today&lt;&lt;</a></td></tr></table></td></tr></table>
        _EOT_
    ($newsTable, $categoryLinks) = parseNews ($newsPage);
    $output .= $newsTable;
    ($output .= <<"        _EOT_") =~ s/^\s+//gm;
        <table class="date"><tr><td class="date">
            $newsDateToday <a href="index.htm" class="date">show torrents</a>
        </td></tr></table>
        _EOT_
    $output =~ s/<!--CATLINKS-->/$categoryLinks/sg;
    savePage ("$conf{htmldir}/yesterday.htm", $output, 1);
    $torrentsYesterday = () = $output =~ /<a href="[^"]+\.torrent"/g;
    open (IN, "< $conf{htmldir}/info.htm") || dieNice "could not open $conf{htmldir}/info.htm: $!\n";
    $info .= $_ while (<IN>);
    close IN;
    my $generated = strftime ('%d-%m-%Y, %H:%M ', localtime (time)) . $conf {'timezone'};
    my $updateString;
    if ($conf {'updatecheck'} eq 'yes') {
        if ($currentVersion < $latestVersion) {
            $updateString = '<a href="http://en.suprnova.ntworks.net/">there is a new version available</a>';
        }
        else { $updateString = 'your version is up to date' }
    }
    else { $updateString = 'checking for updates disabled' }
    $re {'<!--UPDNEWS-->.*<!--/UPDNEWS-->'}
        = "<!--UPDNEWS-->$conf{newtorrentsupdates}<!--/UPDNEWS-->";
    $re {'<!--NUPDATE-->.*<!--/NUPDATE-->'}       = "<!--NUPDATE-->$generated<!--/NUPDATE-->";
    $re {'<!--NMIRROR-->.*<!--/NMIRROR-->'}       = "<!--NMIRROR-->$mirror<!--/NMIRROR-->";
    $re {'<!--NTODAY-->.*<!--/NTODAY-->'}         = "<!--NTODAY-->$torrentsToday<!--/NTODAY-->";
    $re {'<!--NYESTERDAY-->.*<!--/NYESTERDAY-->'} = "<!--NYESTERDAY-->$torrentsYesterday<!--/NYESTERDAY-->";
    $re {'<!--NUPDMIRROR-->.*<!--/NUPDMIRROR-->'} = "<!--NUPDMIRROR-->$newsCreated<!--/NUPDMIRROR-->";
    unless (exists $cmdln {'n'}) {
        $re {'<!--AMIRROR-->.*<!--/AMIRROR-->'} = "<!--AMIRROR-->$mirror<!--/AMIRROR-->";
        $re {'<!--AUPDATE-->.*<!--/AUPDATE-->'} = "<!--AUPDATE-->$generated<!--/AUPDATE-->";
        $re {'<!--ACATEGORIES-->.*<!--/ACATEGORIES-->'} =
            "<!--ACATEGORIES-->$numberCategories<!--/ACATEGORIES-->";
        $re {'<!--ATORRENTS-->.*<!--/ATORRENTS-->'} = "<!--ATORRENTS-->$numberTorrents<!--/ATORRENTS-->";
        $re {'<!--UPDCAT-->.*<!--/UPDCAT-->'} =
            "<!--UPDCAT-->$conf{categoriesupdates}<!--/UPDCAT-->";
    }
    $re {'<!--SNLUPDATE-->.*<!--/SNLUPDATE-->'} = "<!--SNLUPDATE-->$updateString<!--/SNLUPDATE-->";
    $info =~ s/$_/$re{$_}/sgi foreach (keys %re);
    open (OUT, "> $conf{htmldir}/info.htm") || dieNice "could not open $conf{htmldir}/info.htm: $!\n";
    print OUT $info;
    close OUT;
}

sub getTemplate {
    my $templ;
    $templ .= $_ while (<DATA>);
    $templ;
}

sub backupMirror {
    my $file = shift || dieNice "no mirror file specified\n";
    my ($fastest, $lastAge, $fastestURL) = (0, 0, '');
    print "checking for latest and fastest mirror...\n" if $conf {'output'} eq 'yes';
    my @old;
    open (IN, "< $file") || dieNice "cannot open $file for reading: $!";
    push @old, $_ while (<IN>);
    close IN;
    open (OUT, "> $file") || dieNice "cannot open $file for writing: $!";
    foreach (@old) {
        chomp (my $mirror = $_);
        my $start = [gettimeofday];
        my $page = getPage ($mirror . "list_news.html");
        my $time = tv_interval $start;
        my $speed = length ($page) / $time / 1024;
        $page =~ /Generated @ (\d{2})-(\d{2})-(\d{4}) \d{2}:\d{2} /;
        my $age = "$3$2$1";
        if ($age ne '') {
            print OUT $mirror, "\n" if length ($page) > 0;
            if ($age >= $lastAge && length ($page) > 4000) {
                if ($speed > $fastest) {
                    $fastest = $speed;
                    $fastestURL = $mirror;
                }
                printf "%6d B% 8.2f KB/s %8d %s\n", length ($page), $speed, $age, $mirror
                    if $conf {'output'} eq 'yes';
                $lastAge = $age;
            }
            elsif ($conf {'output'} eq 'yes') {
                printf "%6d B% 8.2f KB/s %8d %s (bad mirror)\n", length ($page), $speed, $age, $mirror;
            }
        }
    }
    close OUT;
    return $fastestURL;
}

sub getPage {
    my $url = shift || return undef;
    print "retrieving $url ...\n" if $conf {'output'} eq 'yes';
    my $ua = LWP::UserAgent -> new (
        agent      => 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.7.5) Gecko/20041109 Firefox/1.0',
        timeout    => 20,
        keep_alive => 1,
    );
    $ua -> proxy ('http', $conf {'proxy'}) if (defined $conf {'proxy'});
    my $response = $ua -> get ($url);
    if ($response -> is_error) { return undef }
    else { return $response -> content }
}

sub getMirror {
    my $mirror;
    if (exists $cmdln {'m'}) {
        print "using specified mirror: $cmdln{m}\n" if $conf {'output'} eq 'yes';
        $mirror = $cmdln {'m'};
    }
    elsif (exists $cmdln {'b'}) { $mirror = backupMirror ("$conf{basedir}/$conf{mirrorfile}") }
    else {
        print "checking for random mirror...\n" if $conf {'output'} eq 'yes';
        my $suprnovaMain = getPage ('http://www.suprnova.org/');
        if ($suprnovaMain =~ /<frame name='main' src=['"]?([^'">]+)['"]?.*>/sg ) {
            my ($currentMirror, $found, $fileName) = ($1, 0, "$conf{basedir}/$conf{mirrorfile}");
            my @old;
            open (IN, "< $fileName") || dieNice "cannot open $fileName for reading: $!\n";
            push @old, $_ while (<IN>);
            close IN;
            open (OUT, "> $fileName") || dieNice "cannot open $fileName for writing: $!\n";
            foreach (@old) {
                chomp;
                print OUT "$_\n";
                $found = 1 if ($currentMirror eq $_);
            }
            print OUT "$currentMirror\n" if ($found == 0);
            close OUT;
            $mirror = $currentMirror;
        }
        else { $mirror = backupMirror ("$conf{basedir}/$conf{mirrorfile}") }
    }
    print "using mirror $mirror\n" if $conf {'output'} eq 'yes';
    $mirror;
}

sub getCommandline () { # -n -b -m <mirror> -c <config file>
    my %cmdline;
    getopts ('nbm:c:', \%cmdline);
    $cmdline {'n'} = 1 if exists $cmdline {'n'};
    $cmdline {'b'} = 1 if exists $cmdline {'b'};
    dieNice "you can't use -m and -b at the same time\n"
        if (exists $cmdline {'b'} && exists $cmdline {'m'});
    dieNice "you need to specify the mirror to use with -m\n"
        if (exists $cmdline {'m'} && ! defined $cmdline {'m'});
    dieNice "you need to specify the config file to use with -c\n"
        if (exists $cmdline {'c'} && ! defined $cmdline {'c'});
    dieNice "config file '$cmdline{c}' does not exist\n"
        if (exists $cmdline {'c'} && ! -f $cmdline {'c'});
    %cmdline;
}

sub getConfiguration {
    my $file = shift || 'snmirror.conf';
    my %conf;
    open (CONF, $file) || dieNice ("could not open configuration file $file: $!\n");
    while (<CONF>) {
        chomp;
        if ((! /^#/) && ($_)) {
            /\s*(\S*)=(.*)/;
            $conf {$1} = $2;
        }
    }
    close CONF;
    dieNice "basedir '$conf{basedir}' does not exist\n"
        unless -d $conf {'basedir'};
    dieNice "htmldir '$conf{htmldir}' does not exist\n"
        unless -d $conf {'htmldir'};
    unless (-f "$conf{basedir}/$conf{mirrorfile}") {
        print "mirror file '$conf{basedir}/$conf{mirrorfile}' does not exist..."
            if $conf {'output'} eq 'yes';
        open (MIRROR, "> $conf{basedir}/$conf{mirrorfile}") || dieNice "could not be created: $!\n";
        close MIRROR;
        print "created\n" if $conf {'output'} eq 'yes';
    }
    unless (-f "$conf{basedir}/$conf{categoryfile}") {
        print "category file '$conf{basedir}/$conf{categoryfile}' does not exist..."
            if $conf {'output'} eq 'yes';
        open (CONF, "> $conf{basedir}/$conf{categoryfile}") || dieNice "could not be created: $!\n";
        close CONF;
        print "created\n" if $conf {'output'} eq 'yes';
    }
    unless (-f "$conf{basedir}/$conf{categoryconf}") {
        print "category configuration '$conf{basedir}/$conf{categoryconf}' does not exist..."
            if $conf {'output'} eq 'yes';
        open (CATCONF, "> $conf{basedir}/$conf{categoryconf}") || dieNice "could not be created: $!\n";
        close CATCONF;
        print "created\n" if $conf {'output'} eq 'yes';
    }
    %conf;
}

sub getCategoryConfiguration {
    my %catConf;
    open (CATCONF, $conf {'categoryconf'}) || dieNice ("could not open configuration file: $!\n");
    while (<CATCONF>) {
        chomp;
        if ((! /^#/) && ($_)) {
            /(\d*)=(1|0)\t/;
            $catConf {$1} = $2;
        }
    }
    close CONF;
    %catConf;
}

sub checkSNLLatestVersion {
    my $usContent = getPage ("http://suprnova.ntworks.net/snlupdate");
    $usContent;
}

__END__

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head><title>suprnova light &sup2;</title>
<link rel="stylesheet" type="text/css" media="screen" href="style.css" />
<link rel="shortcut icon" href="favicon.ico" /><meta name="robots" content="noindex,nofollow,noarchive" />
<!--JSCRIPT--></head><body><a name="top" id="top"></a>
<table class="header"><tr><td class="header">suprnova light <span class="top">2</span></td>
<td class="generated"><!--GENERATED--></td></tr><tr class="nav">
<td class="nav"><a href="index.htm">New Torrents</a><a href="categories.htm">Categories</a>
<a href="search.cgi">Search</a><a href="info.htm">Status</a></td></tr><tr class="note"><td colspan="2">
<b>Note:</b> This is neither an official suprnova.org mirror, nor is it intended for public use.
<a href="notes.htm">Read more about suprnova light &sup2;...</a></td></tr></table>
<!--CONTENT-->
<div class="copyright">Copyright &copy; 2004 by <a href="http://suprnova.ntworks.net/">ntworks.net</a> - The
<a href="http://bitconjurer.org/BitTorrent/">BitTorrent</a> protocol and original client and tracker written
by Bram Cohen.</div></body></html>

<!-- Copyright (C) 2004 by ntworks.net - http://suprnova.ntworks.net/ -->
